<?php

include_once("sanitize-functions.php");

/*
 * the following fields are variables passed to sql queries
 * that should always be numeric.
 *
 */
$debsec_number_vars = array(
	"autoscaleopts", "basevalue", "branch", "branch_id",
	"cdeffunctionid", "cdefid", "colorid", "columnnumber",
	"consolidationfunction", "data_source_id", "datasource_id",
	"datasourcetypeid", "did", "dsid", "dstypeid", "fid", "gid",
	"graph_height", "graphid", "graph_id", "graphpolicy",
	"graph_start", "graph_template_id", "graphtypeid", "graph_width",
	"heartbeat", "height", "hide", "hostid", "id", "imageformatid",
	"isparent", "listviewtype", "loginopts", "lowerlimit",
	"maxvalue", "menuid", "minvalue", "pagerefresh", "parent",
	"rigid", "rra", "sequenceparent", "source_id",
	"srcid", "step", "subdsid_old", "subfieldid", "tid", "timespan",
	"treeid", "tree_id", "upperlimit", "userid", "user_id",
	"viewtype", "width"
	); 

/*
 * the following fields are variables psased to sql queries
 * that must meet a certain regex if they are set.
 *
 */
$debsec_regex_vars = array(
	"active" => "on", "autopadding" => "on", "autoscale" => "on",
	"currentds" => "on", "export" => "on", "graphsettings" => "on",
	"grouping" => "on", "hardreturn" => "on", 
	"mustchangepassword" => "on", "showlist" => "on", 
	"showpreview" => "on", "showtree" => "on", "updaterra" => "on", 
	"ip" => "^[0-9.]*$", "hostname" => "^[a-zA-Z0-9.-_]*$", 
	"inputoutput" => "^(in|out)$", "rraid" => "^([0-9]*|all)$"
	);

/*
 * the following variables are user-supplied or other forms of
 * free text, and are passed inside of quotes to sql queries
 *
 */
$debsec_textq_vars = array(
	"custom", "dataname", "dsname", "dspath", "fullname", "name",
	"password_to_save", "section", "textformat", "title", "type",
	"username", "value", "verticallabel", "password", "confirm",
	"snmp_community", "snmp_ip"
	);



/*
 * this will quit out if any fields are set and not numeric
 *
 */
foreach($debsec_number_vars as $debsec_v){
	if(isset($_GET[$debsec_v])){
		input_validate_input_number($_GET[$debsec_v]);
	}
	if(isset($_POST[$debsec_v])){
		input_validate_input_number($_POST[$debsec_v]);
	}
	if(isset($_COOKIE[$debsec_v])){
		input_validate_input_number($_COOKIE[$debsec_v]);
	}
};


/*
 * this will quit out if any fields are set and don't match
 * the given regular expressions
 *
 */
foreach($debsec_regex_vars as $debsec_k => $debsec_re){
	if(isset($_GET[$debsec_k])){
		input_validate_input_regex($_GET[$debsec_k], $debsec_re);
	}
	if(isset($_POST[$debsec_k])){
		input_validate_input_regex($_POST[$debsec_k], $debsec_re);
	}
	if(isset($_COOKIE[$debsec_k])){
		input_validate_input_regex($_COOKIE[$debsec_k], $debsec_re);
	}
};

/*
 * this will clean up all of the above listed quoted text fields
 *
 */
foreach($debsec_textq_vars as $debsec_v){
	if(isset($_GET[$debsec_v])){
		$_GET[$debsec_v] = ereg_replace('"', '\\"', $_GET[$debsec_v]);
	}
	if(isset($_POST[$debsec_v])){
		$_POST[$debsec_v] = ereg_replace('"', '\\"', $_POST[$debsec_v]);
	}
	if(isset($_COOKIE[$debsec_v])){
		$_COOKIE[$debsec_v] = ereg_replace('"', '\\"', $_COOKIE[$debsec_v]);
	}
	global $$debsec_v;
	if(isset($$debsec_v)){
		$$debsec_v = ereg_replace('"', '\\"', $$debsec_v);
	}
};

?>
